import {AdEntity, NewAdEntity, SimpleAdEntity} from "../types";
import {ValidateError} from "../utils/errors";
import {pool} from "../utils/db";
import {FieldPacket} from "mysql2";
import {v4 as uuid} from 'uuid'

type AdRecordResults = [AdEntity[], FieldPacket[]];

export class AdRecord implements AdEntity {
    id: string;
    name: string;
    description: string;
    price: number;
    url: string;
    lat: number;
    lon: number;
    constructor(obj: NewAdEntity) {
        const {id, name, description, price, url, lat, lon} = obj;

        if (!name || name.length > 100) {
            throw new ValidateError('Nazwa ogłoszenia nie może być pusta, ani przekraczać 100 znaków.');
        }

        if (description.length > 1000) {
            throw new ValidateError('Treść ogłoszenia nie może przekraczać 1000 znaków.');
        }

        if (price < 0 || price > 9999999) {
            throw new ValidateError('Cena nie może być mniejsza niż 0 lub większa niż 9 999 999');
        }

        if (!url || url.length > 100) {
            throw new ValidateError('Link ogłoszenia nie może być pusty, ani przekraczać 100 znaków.');
        }

        if (typeof lat !== 'number' || typeof lon !== 'number') {
            throw new ValidateError('Nie można zlokalizować ogłoszenia.');
        }

        this.id = id;
        this.name = name;
        this.description = description;
        this.price = price;
        this.url = url;
        this.lat = lat;
        this.lon = lon;
    }
    static async getOne(id: string): Promise<AdRecord | null> {
        const [results] = await pool.execute('select * from ads where id = :id', {
            id,
        }) as AdRecordResults;

        return results.length === 0 ? null : new AdRecord(results[0]);
    }
    static async findAll(name: string): Promise<SimpleAdEntity[]> {
        const [results] = await pool.execute('select * from ads where name like :search', {
            search: `%${name}%`,
        }) as AdRecordResults;

        return results.map(result => {
            const { id, lat, lon } = result;

            return {
                id, lat, lon
            };
        });
    }

    async insert(): Promise<void> {
        if (!this.id) {
            this.id = uuid();
        } else {
            throw new Error('Cannot inserted something that is already inserted!')
        }

        await pool.execute('insert into ads (id, name, description, price, url, lat, lon) ' +
            'values (:id, :name, :description, :price, :url, :lat, :lon)', this);
    }

}
